/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "project.h"
#include <stdio.h>
#include <stdbool.h>

void InitialiseSystem(); 
void blink();
CY_ISR_PROTO(WDT_Int_ISR);
void InitWatchdog(); 
void AlarmCallbackHandler(); 
void RTCSetTime();
void RTCSetAlarm(); 

uint32 AlarmMask = RTC_ALARM_SEC_MASK | RTC_ALARM_MIN_MASK | RTC_ALARM_HOUR_MASK;   // works one time 
// uint32 AlarmMask = RTC_ALARM_SEC_MASK | RTC_ALARM_MIN_MASK ;     // Does not work
// uint32 AlarmMask = RTC_ALARM_SEC_MASK ;  // Does not work 
uint8 ShowTempCounter = 0;  // Watchdog-Counter

// Time-Definition
uint32 Day = 26;    
uint32 Month = 4; 
uint32 Year = 2018; 
uint32 Hour = 21; 
uint32 Minutes = 05; 
uint32 Seconds = 55; 
uint32 AlarmMinutes = 6; 
uint32 AlarmSeconds = 0; 

int main(void)
{
    InitialiseSystem();
    
    for(;;)
    {        
        // Reset Watchdog Counter
        ShowTempCounter = 0;
        // Sleep until the next interrupt wakes us up. 
        CySysPmSleep();
    }     
}


void InitialiseSystem(){
    blink(); 
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    // Set Watchdog-Interrupt Callback
    wdtint_StartEx(WDT_Int_ISR);
    // Configure the clocks for the watchdog. 
    InitWatchdog();    
    
    // Set Alarm-Mask, values see above 
    RTC_SetAlarmMask( AlarmMask );   
    // Set the Callback-Handler to handle the alarm 
    RTC_SetAlarmHandler( AlarmCallbackHandler );     
    
    // RTC is manually updated on Counter1-Interrupt 
    RTC_SetPeriod(1,1);
    // Start the RTC
    RTC_Start();      
    
    // Set the time
    RTCSetTime();
    // Set the Alarm 
    RTCSetAlarm(); 
}

void AlarmCallbackHandler(){
    // Clear the Alarm-Flag
    RTC_ClearAlarmStatus();
    // Toggle LED 
    LED_Write( ~LED_Read() );     
}
    
// Set the time to the predefined values 
void RTCSetTime(){
    uint32 InputTime = 0; 
    InputTime = RTC_SetHours( InputTime, Hour ); 
    InputTime = RTC_SetMinutes( InputTime, Minutes );
    InputTime = RTC_SetSecond( InputTime, Seconds ); 
    uint32 InputDate = 0; 
    InputDate = RTC_SetYear( InputDate, Year ); 
    InputDate = RTC_SetMonth( InputDate, Month );
    InputDate = RTC_SetDay( InputDate, Day );
    RTC_SetDateAndTime( InputTime, InputDate );     
}


// Set the Alarm 5s later than the current time 
void RTCSetAlarm() {
    
    RTC_DATE_TIME MyAlarm; 
    RTC_DATE_TIME testReadback;    
    
    MyAlarm.date = RTC_SetDay( MyAlarm.date, Day ); 
    MyAlarm.date = RTC_SetMonth( MyAlarm.date, Month ); 
    MyAlarm.date = RTC_SetYear( MyAlarm.date, Year ); 
    MyAlarm.time = RTC_SetHours( MyAlarm.time, Hour );
    MyAlarm.time = RTC_SetMinutes( MyAlarm.time, AlarmMinutes );
    MyAlarm.time = RTC_SetSecond( MyAlarm.time, AlarmSeconds );
 
    // Set the Alarm
    RTC_SetAlarmDateAndTime( &MyAlarm );
    // Read back the Alarm to verify everything is ok.
    RTC_GetAlarmDateAndTime( &testReadback ); 
    
    if( ( testReadback.date == 0 ) & ( testReadback.time == 0) )  {
        // If date and time is zero, this means Set-Alarm failed. 
        
        // Endless loop, toggle LED fast and end up.  
        while(1){
            LED_Write( ~LED_Read() ); 
            CyDelay(100); 
        }            
    }
}

// This is the Watchdog-Interrupt 
CY_ISR(WDT_Int_ISR)
{
    uint32 InterruptSource = CySysWdtGetInterruptSource();
    
    /* clear interrupt flag to enable next interrupt */
    CySysWdtClearInterrupt(InterruptSource);     
    
    switch(InterruptSource)
    {
        case CY_SYS_WDT_COUNTER0_INT:   // 23ms
            // This one is to wake up for Handle BLE Events (in real application)
            break;
           
        case CY_SYS_WDT_COUNTER1_INT:   // 1s
            RTC_Update();               // Update of RTC
            ShowTempCounter++;
            if( ShowTempCounter > 5 ) CySoftwareReset();
            break;
            
        default:
            break;
    }
}

// Just set the Clear on Match to both Watchdog-Counters
void InitWatchdog()
{
    // basic function is defined in clock-tab of cydwr
    
    // 23ms BLE-Timer
    CySysWdtWriteMode(CY_SYS_WDT_COUNTER0, CY_SYS_WDT_MODE_INT);
	CySysWdtWriteMatch(CY_SYS_WDT_COUNTER0, 444u);
	CySysWdtWriteClearOnMatch(CY_SYS_WDT_COUNTER0, 1u);
	CySysWdtEnable(CY_SYS_WDT_COUNTER0_MASK); 
    
    // 1s RTC Timer
    CySysWdtWriteMode(CY_SYS_WDT_COUNTER1, CY_SYS_WDT_MODE_INT);
	CySysWdtWriteMatch(CY_SYS_WDT_COUNTER1, 32768u);
	CySysWdtWriteClearOnMatch(CY_SYS_WDT_COUNTER1, 1u);
	CySysWdtEnable(CY_SYS_WDT_COUNTER1_MASK);

    CyDelay(3); // Just to make sure the settings come in effect
}
   
void blink() {
    // Debug-feature to signal the start of the software.
    uint8 i =0;
    for(i=0;i<4;i++)
    {
        LED_Write(1);
        CyDelay(100);
        LED_Write(0);
        CyDelay(100);
    }    
}



/* [] END OF FILE */
